# Generated by Django 2.1.14 on 2020-02-11 10:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0047_auto_20200207_1126'),
    ]

    operations = [
        migrations.AlterField(
            model_name='timeentry',
            name='hours_to_bill',
            field=models.DecimalField(blank=True, decimal_places=4, max_digits=9, null=True),
        ),
        migrations.AlterField(
            model_name='timeentry',
            name='hours_worked',
            field=models.DecimalField(blank=True, decimal_places=4, max_digits=9, null=True),
        ),
        migrations.AlterField(
            model_name='timeentry',
            name='offset_hours',
            field=models.DecimalField(blank=True, decimal_places=4, max_digits=9, null=True),
        ),
    ]
