# Generated by Django 2.1.11 on 2019-11-27 17:18

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djautotask', '0029_phase_last_activity_date'),
    ]

    operations = [
        migrations.AddField(
            model_name='task',
            name='phase',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='djautotask.Phase'),
        ),
    ]
